#!/usr/bin/env wish

#******************************
# 
# osOTP 
# Version 2021-09-25
# Copyright (C) 2015-2021 Oliver Scholl < tcltk@posteo.de > < www.tcltk.info >
#
# Requirement:
# tcl 8.6 or higher
# tk 8.6 or higher
#
# Description of the program:
# This is a cryptography progrogram that uses the one-time-pad encryption technique.
# The program encrypts arbitrary files or text using a key file. The key file 
# must be at least as long as the input file or the text length. If you don't have a key file it is possible
# to create a key file with (pseudo-)random numbers by the program itself. Please make sure not to 
# delete the key file otherwise you can't decode your encrypted file or text anymore.
# For further information about one-time-pad and true randomness you should search 
# the internet (e. g. http://en.wikipedia.org/wiki/One-time_pad).
# Please pay attention to the cryptography laws in your country.
#
# License:
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 2 of 
# the license, or any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# merchantability or fitness for a particular purpose. 
# See the GNU General Public License for more details.
#
#*************************************

set tclexecomp 0 ; # 1=wenn das Programm mit tclexecomp benutzt wird. Sonst 0.

package require msgcat
package require tooltip

# Workaround fuer tclexecomp
namespace inscope :: [list ::source -encoding utf-8  [file join [file dirname [info script]] de.msg]]
namespace inscope :: [list ::source -encoding utf-8  [file join [file dirname [info script]] en.msg]]

set Schriftgroesse 12
	font create GlobSchrift -family Helvetica -size $Schriftgroesse -slant roman -weight normal
	ttk::style theme use default
	ttk::style configure TButton -font GlobSchrift
	ttk::style configure TLabel -font GlobSchrift
	ttk::style configure TLabelFrame -font GlobSchrift 
	ttk::style configure TMenubutton -font GlobSchrift 
	ttk::style configure TRadiobutton -font GlobSchrift
	ttk::style configure TEntry -font GlobSchrift 
	ttk::style configure TCombobox -font GlobSchrift 
	ttk::style configure TCheckbutton -font GlobSchrift
	ttk::style configure TNotebook.Tab -font GlobSchrift 
	ttk::style configure Treeview -font GlobSchrift 
	ttk::style configure Treeview.Heading -font GlobSchrift
	option add *TCombobox*Listbox.font GlobSchrift
	option add *Dialog.msg.font GlobSchrift


set ProgrammName "osOTP"
set ProgrammVersion "2021-09-25"
set ProgrammTitel "One-Time-Pad"

set Konf(Skriptname) [info script]
if {[file type $Konf(Skriptname)] == "link"} {
	set Konf(Skriptname) [file readlink $Konf(Skriptname)]
}
::msgcat::mclocale de ;# Alternative Sprachen: de=deutsch en=englisch
::msgcat::mcload [file join [file dirname $Konf(Skriptname)]]

wm protocol . WM_DELETE_WINDOW {
	# wird beim Schliessen des Hauptfensters ausgefuehrt
	ProgrammBeenden
}

proc ProgrammBeenden {} {
	global Konfigurationsdatei
	global EingabeDateiText	
	global AusgabeDateiText
	global Eingabedatei
	global Schluesseldatei
	global HistogrammJaNein
	global Sprache
	global Schriftgroesse
	global Schluesselliste
	
	if {[catch {set f [open $Konfigurationsdatei w]}] == 0} {
		puts $f "1:$EingabeDateiText"
		puts $f "2:$AusgabeDateiText"
		puts $f "3:$Eingabedatei"
		puts $f "4:$Schluesseldatei"
		puts $f "5:$HistogrammJaNein"
		puts $f "6:$Sprache"
		puts $f "7:$Schriftgroesse"
		foreach Element $Schluesselliste {
			puts $f "8:$Element"		
		}
		close $f
	}
	exit
}

proc SpracheAendern {Sprache} {
	global Konf

	switch $Sprache {
		Deutsch {::msgcat::mclocale de}
		English {::msgcat::mclocale en}
	}
	::msgcat::mcload [file join [file dirname $Konf(Skriptname)]]
	
	.frAlles.frEingabe.rbEingabeDatei configure -text [::msgcat::mc "Datei"]
	.frAlles.frEingabe.rbEingabeText configure -text [::msgcat::mc "Text"]
	.frAlles.frAusgabe.rbAusgabeDatei configure -text [::msgcat::mc "Datei"]
	.frAlles.frAusgabe.rbAusgabeText configure -text [::msgcat::mc "Text"]
	.frAlles.lb1Eingabedatei configure -text [::msgcat::mc "Datei:"]
	.frAlles.lbSchluesseldatei configure -text [::msgcat::mc "Schluessel:"]
	.frAlles.btSchluesselErzeugen configure -text [::msgcat::mc "Erzeugen"]
	.frAlles.lbStartwert configure -text [::msgcat::mc "Starte bei Byte:"]
	.frAlles.frHistogramm.rbHistogrammJa configure -text [::msgcat::mc "ja"]
	.frAlles.frHistogramm.rbHistogrammNein configure -text [::msgcat::mc "nein"]	
	.frAlles.btStart configure -text [::msgcat::mc "Start"]
	.frAlles.lb1Meldung configure -text [::msgcat::mc "Meldung:"]
	.frAlles.lbFortschritt configure -text [::msgcat::mc "Fortschritt:"]
	.frAlles.lbText configure -text [::msgcat::mc "Text:"]
	.frAlles.lbSuchen configure -text [::msgcat::mc "Suchbegriff:"]
	.frAlles.btSuchen configure -text [::msgcat::mc "Suchen"]
	
	.m entryconfigure 0 -label [::msgcat::mc "Schliessen"]
	.m entryconfigure 1 -label [::msgcat::mc "Sprache"]
	.m entryconfigure 2 -label [::msgcat::mc "Schriftgroesse"]
	.m entryconfigure 3 -label [::msgcat::mc "Info"]

	::tooltip::tooltip .frAlles.frEingabe.rbEingabeDatei [::msgcat::mc "Verschluesselt oder entschluesselt eine Datei."]
	::tooltip::tooltip .frAlles.frEingabe.rbEingabeText [::msgcat::mc "Verschluesselt oder entschluesselt den Text in dem unten gezeigten Textfeld."]
	::tooltip::tooltip .frAlles.frAusgabe.rbAusgabeDatei [::msgcat::mc "Verschluesselt oder entschluesselt in eine Datei."]
	::tooltip::tooltip .frAlles.frAusgabe.rbAusgabeText [::msgcat::mc "Verschluesselt oder entschluesselt in das unten gezeigte Textfeld."]
	::tooltip::tooltip .frAlles.lb2Eingabedatei [::msgcat::mc "Diese Datei wird ver- oder entschluesselt."]
	::tooltip::tooltip .frAlles.btEingabedatei [::msgcat::mc "Waehlen Sie eine Datei zum ver- oder entschluesseln aus."]
	::tooltip::tooltip .frAlles.cbSchluesseldatei [::msgcat::mc "Das ist der Schluessel mit dem die Datei ver- oder entschluesselt wird."]
	::tooltip::tooltip .frAlles.btSchluesseldatei [::msgcat::mc "Waehlen Sie einen Schluessel aus."]
	::tooltip::tooltip .frAlles.btSchluesselErzeugen [::msgcat::mc "Wenn Sie noch keinen Schluessel haben,\nkoennen Sie einen Schluessel erzeugen."]
	::tooltip::tooltip .frAlles.enStartwert [::msgcat::mc "Gibt an, an welcher Stelle der Schluessel startet."]
	::tooltip::tooltip .frAlles.btStart [::msgcat::mc "Startet das Ver- oder Entschluesseln."]
	::tooltip::tooltip .frAlles.frText.tx [::msgcat::mc "Das Textfeld hat folgende Funktionen:\n(1) Wenn Sie eine Datei als Text enschluesseln, wird der entschluesselte Inhalt der Datei im Textfeld angezeigt.\n(2) Sie koennen in das Textfeld einen Text eingeben und dann verschluesseln.\n(3) Sie koennen in das Textfeld einen verschluesselten Text hinein kopieren und dann entschluesseln.\nWenn Sie den Cursor vor das erste Zeichen setzen,\nkoennen Sie mit der Tastenkombination Shift+Strg+Ende den gesamten Text markieren,\nmit Strg+c kopieren und mit Strg+v in ein anderes Programm einfuegen."]
	::tooltip::tooltip .frAlles.enSuchen [::msgcat::mc "Geben Sie einen Suchbegriff ein, der in dem Textfeld gesucht werden soll.\nDie Gross-/Kleinschreibung wird beachtet."]
	::tooltip::tooltip .frAlles.btSuchen [::msgcat::mc "Durchsucht das Textfeld nach dem Suchbegriff.\nDie Gross-/Kleinschreibung wird beachtet."]
	
	update
}

proc InfoAnzeigen {ProgrammName ProgrammTitel ProgrammVersion} {
	set Titel [::msgcat::mc "Info"]
	set Text "${ProgrammName} ${ProgrammTitel}\nVersion ${ProgrammVersion}\n\nCopyright (C) Oliver Scholl\nwww.tcltk.info\n
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the license, or any later version.\n
This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of merchantability or fitness for a particular purpose. See the GNU General Public License for more details."
	tk_messageBox -title $Titel -message $Text -type ok
}

proc DateiTextAuswaehlen {Eingabe Ausgabe} {
	# Klick auf die Radiobuttons Datei oder Text fuer die Eingabe oder Ausgabe
	if {$Eingabe eq "Datei" && $Ausgabe eq "Datei"} {
		.frAlles.lb1Eingabedatei configure -state normal
		.frAlles.lb2Eingabedatei configure -state normal
		.frAlles.btEingabedatei configure -state normal
		#.frAlles.btSchluesselErzeugen configure -state normal
		grid forget .frAlles.lbText
		grid forget .frAlles.frText
		grid forget .frAlles.lbSuchen
		grid forget .frAlles.enSuchen
		grid forget .frAlles.btSuchen
	}
	if {$Eingabe eq "Datei" && $Ausgabe eq "Text"} {
		.frAlles.lb1Eingabedatei configure -state normal
		.frAlles.lb2Eingabedatei configure -state normal
		.frAlles.btEingabedatei configure -state normal
		#.frAlles.btSchluesselErzeugen configure -state normal
		grid .frAlles.lbText -row 9 -column 0 -padx 3 -pady 3 -sticky nw
		grid .frAlles.frText -row 9 -column 1 -columnspan 3 -sticky nesw
		grid .frAlles.lbSuchen -row 10 -column 0 -padx 3 -pady 3 -sticky w
		grid .frAlles.enSuchen -row 10 -column 1 -padx 3 -pady 3 -sticky we
		grid .frAlles.btSuchen -row 10 -column 2 -padx 3 -pady 3 -sticky w
	}	

	if {$Eingabe eq "Text" && $Ausgabe eq "Datei"} {
		.frAlles.lb1Eingabedatei configure -state disabled
		.frAlles.lb2Eingabedatei configure -state disabled
		.frAlles.btEingabedatei configure -state disabled
		#.frAlles.btSchluesselErzeugen configure -state disabled
		grid .frAlles.lbText -row 9 -column 0 -padx 3 -pady 3 -sticky nw
		grid .frAlles.frText -row 9 -column 1 -columnspan 3 -sticky nesw
		grid .frAlles.lbSuchen -row 10 -column 0 -padx 3 -pady 3 -sticky w
		grid .frAlles.enSuchen -row 10 -column 1 -padx 3 -pady 3 -sticky we
		grid .frAlles.btSuchen -row 10 -column 2 -padx 3 -pady 3 -sticky w
	}	

	if {$Eingabe eq "Text" && $Ausgabe eq "Text"} {
		.frAlles.lb1Eingabedatei configure -state disabled
		.frAlles.lb2Eingabedatei configure -state disabled
		.frAlles.btEingabedatei configure -state disabled
		#.frAlles.btSchluesselErzeugen configure -state disabled
		grid .frAlles.lbText -row 9 -column 0 -padx 3 -pady 3 -sticky nw
		grid .frAlles.frText -row 9 -column 1 -columnspan 3 -sticky nesw
		grid .frAlles.lbSuchen -row 10 -column 0 -padx 3 -pady 3 -sticky w
		grid .frAlles.enSuchen -row 10 -column 1 -padx 3 -pady 3 -sticky we
		grid .frAlles.btSuchen -row 10 -column 2 -padx 3 -pady 3 -sticky w
	}	
}

proc TextPruefen {} {
	# Prueft, ob in dem Textfeld Zeichen vorkommen, die einen Dezimalwert > 255 haben
	set Text [.frAlles.frText.tx get 1.0 end]
	set Textposition 0
	while {1} {
		set Zeichen [string index $Text $Textposition]
		incr Textposition
		if {$Zeichen eq ""} {
			break
		}
		set ZeichenDezimal [scan $Zeichen %c]
		if {$ZeichenDezimal > 255} {
			tk_messageBox -title [::msgcat::mc "Hinweis"] -message [::msgcat::mc "Der Text enthaelt Zeichen, die nicht im ASCII-Zeichensatz enthalten sind.\nDiese Zeichen werden beim Verschluesseln in eine Tilde konvertiert."] -icon info -type ok -default ok
			break
		}
	}
}

proc OTP {Fall Aktion Eingabedatei Schluesseldatei Ausgabedatei Startwert Histogramm Konsole} {
	# Ver-/Entschluesselt die Datei bzw. den Text.
	# Die Variable "Eingabedatei" enthaelt entweder den Dateinamen oder den Text.
	# Startwert = gibt an, ab welchem Byte der Schluessel verwendet werden soll

	if {$Eingabedatei eq "" || $Schluesseldatei eq ""} {
		return
	}

	if {$Fall == 1 || $Fall == 3} { 
		if {$Ausgabedatei eq ""} {
			return
		}
	}

	if {$Fall == 2 || $Fall == 4} {
		catch {.frAlles.frText.tx delete 0.0 end}
		update idletasks
	}

	# fuer Histogramm. Zaehlt, wie haeufig der Schluessel die Werte 0 bis 255 benutzt.
	for {set i 0} {$i <= 255} {incr i} {
		set Anzahl($i) 0
	}

	if {$Fall == 1 || $Fall == 2} {
		set FortschrittGrenze [expr [file size $Eingabedatei]/100]
	} else {
		set FortschrittGrenze [expr [string length $Eingabedatei]/100]
	}
	set FortschrittZaehler 0
	set FortschrittZaehler2 0	

	# Ver-/Entschluesselung durchfuehren
	if {$Fall == 1 || $Fall == 2} {
		set fEingabe [open $Eingabedatei r]
		fconfigure $fEingabe -translation binary
	}
	
	set fSchluessel [open $Schluesseldatei r]
	fconfigure $fSchluessel -translation binary

	if {$Fall == 1 || $Fall == 3} {
		set fAusgabe [open $Ausgabedatei w]
		fconfigure $fAusgabe -translation binary
	}
	
	set ZaehlerStartwert 0
	while {$ZaehlerStartwert < $Startwert} {
		set Schluessel [read $fSchluessel 1]
		incr ZaehlerStartwert
	}

	set Textposition 0 ; # Position im Text, wenn Text ver-/entschluesselt wird
	while {1} {
		if {$Fall == 1 || $Fall == 2} {
			set Zeichen [read $fEingabe 1]
			if {[eof $fEingabe]} {
				close $fEingabe
				break
			}
		} else {
			set Zeichen [string index $Eingabedatei $Textposition]
			incr Textposition
			if {$Zeichen eq ""} {
				break
			}
			
			if {$Fall == 4 && $Aktion eq "Entschluesseln"} {
				# da es sich um Hexadezimalzeichen handelt,
				# muss noch die zweite Stelle sowie das nachfolgende Leerzeichen eingelesen werden.
				set Zeichen2 [string index $Eingabedatei $Textposition]
				incr Textposition
				if {$Zeichen2 eq ""} {
					break
				}
				set Zeichen3 [string index $Eingabedatei $Textposition]
				incr Textposition
				if {$Zeichen3 eq ""} {
					break
				}
				switch $Zeichen {
					0 {set Zahl 0}
					1 {set Zahl 1}
					2 {set Zahl 2}
					3 {set Zahl 3}
					4 {set Zahl 4}
					5 {set Zahl 5}
					6 {set Zahl 6}
					7 {set Zahl 7}
					8 {set Zahl 8}
					9 {set Zahl 9}
					a - 
					A {set Zahl 10}
					b -
					B {set Zahl 11}
					c -
					C {set Zahl 12}
					d -
					D {set Zahl 13}
					e -
					E {set Zahl 14}
					f -
					F {set Zahl 15}
				}
				switch $Zeichen2 {
					0 {set Zahl2 0}
					1 {set Zahl2 1}
					2 {set Zahl2 2}
					3 {set Zahl2 3}
					4 {set Zahl2 4}
					5 {set Zahl2 5}
					6 {set Zahl2 6}
					7 {set Zahl2 7}
					8 {set Zahl2 8}
					9 {set Zahl2 9}
					a -
					A {set Zahl2 10}
					b -
					B {set Zahl2 11}
					c -
					C {set Zahl2 12}
					d -
					D {set Zahl2 13}
					e -
					E {set Zahl2 14}
					f -
					F {set Zahl2 15}
				}
				set Zeichen [expr 16 * $Zahl + $Zahl2]
				set CodeBinaer [format %08b $Zeichen]
				set Zeichen [binary format B8 $CodeBinaer]
			}
		}
		set Schluessel [read $fSchluessel 1]
		if {[eof $fSchluessel]} {
			puts [::msgcat::mc "Fehler: Der Schluessel ist zu kurz."]
			close $fSchluessel
			if {$Fall == 1 || $Fall == 2} {
				close $fEingabe
			}
			if {$Fall == 1 || $Fall == 3} {
				close $fAusgabe
			}
			break
		}

		set ZeichenDezimal [scan $Zeichen %c]
		if {($Fall == 3 || $Fall == 4) && $Aktion eq "Verschluesseln"} {
			# Falls im Textfeld Zeichen enthalten sind mit einem Bytewert > 255,
			# wird das Zeichen durch ein Leerzeichen ersetzt.
			if {$ZeichenDezimal > 255} {
				set ZeichenDezimal 126 ; # Tilde
			}
		}
		set SchluesselDezimal [scan $Schluessel %c]
		set CodeDezimal [expr $ZeichenDezimal^$SchluesselDezimal]
		set CodeBinaer [format %08b $CodeDezimal]
		set CodeHexadezimal [format %02x $CodeDezimal]

		incr Anzahl($SchluesselDezimal) ; # fuer Histogramm

		# zur Veranschaulichung 
		#set ZeichenBinaer [format %08b $ZeichenDezimal]
		#set SchluesselBinaer [format %08b $SchluesselDezimal]
		#puts "$Zeichen = $ZeichenBinaer | $Schluessel = $SchluesselBinaer | $CodeBinaer"

		set Ausgabe [binary format B8 $CodeBinaer]
		if {$Fall == 1 || $Fall == 3} {
			puts -nonewline $fAusgabe $Ausgabe
		} else {
			if {$Fall == 4 && $Aktion eq "Verschluesseln"} {
				.frAlles.frText.tx insert end [string cat $CodeHexadezimal " "]
			} else {
				.frAlles.frText.tx insert end $Ausgabe
			}
		}
		
		# Fortschrittsbalken aktualisieren
		if {$Konsole == 0} {
			incr FortschrittZaehler
			if {$FortschrittZaehler >= $FortschrittGrenze} {
				incr FortschrittZaehler2
				.frAlles.pb configure -value $FortschrittZaehler2
				update idletask
				set FortschrittZaehler 0
			}
		}
	}

	# Fortschrittsbalken auf 100 Prozent setzen
	if {$Konsole == 0} {
		.frAlles.pb configure -value 100
		update idletask
	}

	if {$Fall == 1 || $Fall == 3} {
		close $fAusgabe
	}
	
	close $fSchluessel

	#HistogrammAnzeigen
	if {$Histogramm == 1} {
		set Hoehe 200
		set Maximum 0
		for {set i 0} {$i <= 255} {incr i} {
			if {$Anzahl($i) > $Maximum} {
				set Maximum $Anzahl($i)
			}
		}
		set Faktor [expr 1.0*$Hoehe/$Maximum]
		toplevel .top4
		wm title .top4 [::msgcat::mc "Histogramm"]
		canvas .top4.c -width 255 -height $Hoehe
		pack .top4.c
		for {set i 0} {$i <= 255} {incr i} {
			set BalkenHoehe [expr round($Hoehe - $Anzahl($i)*$Faktor)]
			.top4.c create line $i $Hoehe $i $BalkenHoehe -width 1 -fill red
		}
		update idletask	
		tkwait visibility .top4
		grab set .top4
	}
}

proc Suchen {Text} {
	# Durchsucht das Text-Widget nach einem Suchbegriff und markiert die Treffer
	.frAlles.frText.tx tag remove Markierung 0.0 end ; # alle tags entfernen
	set Laenge [string length $Text]
	set Liste [.frAlles.frText.tx search -all $Text 1.0]
	set Zaehler 0
	set ErsterTreffer ""
	foreach Element $Liste {
		incr Zaehler
		if {$Zaehler == 1} {
			set ErsterTreffer $Element
		}
		set ZeileSpalte [split $Element "."]
		lassign $ZeileSpalte Zeile Spalte
		set Spalte [expr $Spalte + $Laenge]
		.frAlles.frText.tx tag add Markierung $Element [string cat $Zeile "." $Spalte]
		.frAlles.frText.tx tag configure Markierung -foreground red
	}
	if {$ErsterTreffer ne ""} {
		.frAlles.frText.tx see $ErsterTreffer
	}
}

proc Start {EingabeDateiText AusgabeDateiText Eingabedatei Schluesseldatei Startwert Dateiendung Textkennung HomeOrdner HistogrammJaNein Konsole} {
	# prueft die Eingaben und startet die Ver-/Entschluesselung

	if {$EingabeDateiText eq "Datei" && $AusgabeDateiText eq "Datei"} {
		set Fall 1
	}
	if {$EingabeDateiText eq "Datei" && $AusgabeDateiText eq "Text"} {
		set Fall 2
	}
	if {$EingabeDateiText eq "Text" && $AusgabeDateiText eq "Datei"} {
		set Fall 3
	}
	if {$EingabeDateiText eq "Text" && $AusgabeDateiText eq "Text"} {
		set Fall 4
	}

	set Meldung ""

	if {$Dateiendung eq ""} {
		return
	}

	if {$Startwert eq ""} {
		set Startwert 0
	}

	if {$Fall == 1 || $Fall == 2} {
		if {[file exist $Eingabedatei] == 0} {
			set Meldung [::msgcat::mc "Fehler: Die Datei existiert nicht."]
			return $Meldung
		}

		if {[file isfile $Eingabedatei] == 0} {
			set Meldung [::msgcat::mc "Fehler: Die Datei ist keine gueltige Datei."]
			return $Meldung
		}
		set EingabedateiEndung [string tolower [file extension $Eingabedatei]]
		if {$EingabedateiEndung eq [string tolower $Dateiendung]} {
			set Aktion "Entschluesseln"
		} else {
			set Aktion "Verschluesseln"
		}
	}

	if {$Fall == 3 || $Fall == 4} {
		set ErsteZeile [.frAlles.frText.tx get 1.0 1.end]
		if {[string range $ErsteZeile 0 [expr [string length $Textkennung] - 1]] eq $Textkennung} {
			# es handelt sich um verschluesselten Text
			set Rest [string range $ErsteZeile 5 end]
			if {[string range $Rest 0 5] eq "Start:"} {
				set Rest [string range $Rest 6 end]
				set Index [string first "*" $Rest]
				if {$Index >= 0} {
					set tmpStartwert [string range $Rest 0 [expr $Index - 1]]
					if {$tmpStartwert >= 0} {
						set Startwert $tmpStartwert
					} else {
						set Startwert 0
					}			
				}
			}
			.frAlles.frText.tx delete 1.0 2.0 ; # erste Zeile loeschen
			set Text [.frAlles.frText.tx get 1.0 end]
			set Aktion "Entschluesseln"
		} else {
			set Text [.frAlles.frText.tx get 1.0 end]
			set Aktion "Verschluesseln"
		}
		set Text [string range $Text 0 end-1] ; # loescht das Zeilenende-Zeichen
		set TextLaenge [string length $Text]
		if {$TextLaenge == 0} {
			set Meldung [::msgcat::mc "Fehler: Es wurde kein Text eingegeben."]
			return $Meldung
		}
		TextPruefen
	}

	if {[file exist $Schluesseldatei] == 0} {
		set Meldung [::msgcat::mc "Fehler: Der Schluessel existiert nicht."]
		return $Meldung
	}

	if {[file isfile $Schluesseldatei] == 0} {
		set Meldung [::msgcat::mc "Fehler: Der Schluessel ist keine gueltige Datei."]
		return $Meldung
	}

	set Dateigroesse [file size $Schluesseldatei]
	if {$Fall == 1 || $Fall == 2}  {
		set GroesseEingabe [file size $Eingabedatei]
	} else {
		if {$Aktion eq "Verschluesseln"} {
			set GroesseEingabe $TextLaenge
		} else {
			# Hexadezimalangaben mit nachfolgendem Leerzeichzen
			set GroesseEingabe [expr ($TextLaenge + 1) / 3]
		}
	}
	if {[expr $Startwert + $Dateigroesse] < $GroesseEingabe} {
		set Meldung [::msgcat::mc "Fehler: Der Schluessel ist zu kurz.kurz"]
		return $Meldung
	}
	
	if {$Fall == 1 || $Fall == 2}  {
		set EingabedateiEndung [string tolower [file extension $Eingabedatei]]
		if {$Aktion eq "Verschluesseln"} {
			if {$Fall == 2} {
				set Meldung [::msgcat::mc "Fehler: Eine Datei kann nicht in das Textfeld verschluesselt werden."]
				return $Meldung
			}
			set AktionText [::msgcat::mc "verschluesselt"]
			if {$HistogrammJaNein eq "ja"} {
				set Histogramm 1
			} else {
				set Histogramm 0
			}
			set Ausgabedatei $Eingabedatei$Dateiendung
		} else {
			set AktionText [::msgcat::mc "entschluesselt"]
			set Histogramm 0
			set Ausgabedatei [file rootname $Eingabedatei]
		}
	} else {
		if {$Aktion eq "Verschluesseln"} {
			set AktionText [::msgcat::mc "verschluesselt"]
			if {$HistogrammJaNein eq "ja"} {
				set Histogramm 1
			} else {
				set Histogramm 0
			}
			if {$Fall == 3} {
				set Ausgabedatei [file join $HomeOrdner "[clock clicks -milliseconds].txt.otp"]
			} else {
				set Ausgabedatei ""
			}
		} else {
			set AktionText [::msgcat::mc "entschluesselt"]
			set Histogramm 0
			if {$Fall == 3} {
				set Ausgabedatei [file join $HomeOrdner "[clock clicks -milliseconds].txt"]
			} else {
				set Ausgabedatei ""
			}
		}
	}
	
	if {$Fall == 1 || $Fall == 3} {
		if {[file exist $Ausgabedatei] == 1} {
			set Meldung [::msgcat::mc "Fehler: Die Ausgabedatei %s existiert bereits." $Ausgabedatei]
			return $Meldung
		}
	}
	
	if {$Fall == 3 || $Fall == 4} {
		set Eingabedatei $Text
	}
	
	OTP $Fall $Aktion $Eingabedatei $Schluesseldatei $Ausgabedatei $Startwert $Histogramm $Konsole

	if {$Fall == 1 || $Fall == 2} {
		set Laenge [file size $Eingabedatei]
	} elseif {$Fall == 3} {
		set Laenge [file size $Ausgabedatei]
	} else {
		if {$Aktion eq "Verschluesseln"} {
			set Laenge [string length $Eingabedatei]
		} else {
			set Laenge [.frAlles.frText.tx count -chars 1.0 end]
		}
	}
	
	if {$Fall == 2 || $Fall == 4}  {
		if {$Aktion eq "Verschluesseln"} {
			set Stop [expr $Startwert + [string length $Text] - 1]
			.frAlles.frText.tx insert 1.0 [string cat $Textkennung "Start:" $Startwert "*Stop:" $Stop "*\n"]
		}
	}
	
	if {$Fall == 1 || $Fall == 2} {
		set Stoppwert [expr $Startwert + $Laenge]
		if {$Fall == 1} {
			set Ausgabedatei [file tail $Ausgabedatei]
			set Meldung [::msgcat::mc "Die Datei wurde %s zu: %s. Verbrauchter Schluessel von %s bis %s." $AktionText $Ausgabedatei $Startwert $Stoppwert]
		} else {
			set Meldung [::msgcat::mc "Die Datei wurde %s. Verbrauchter Schluessel von %s bis %s." $AktionText $Startwert $Stoppwert]
		}
	} else {
		set Stoppwert [expr $Startwert + $Laenge]
		if {$Fall == 3} {
			#set Ausgabedatei [file tail $Ausgabedatei]
			set Meldung [::msgcat::mc "Der Text wurde %s zu: %s. Verbrauchter Schluessel von %s bis %s." $AktionText $Ausgabedatei $Startwert $Stoppwert]
		} else {
			set Meldung [::msgcat::mc "Der Text wurde %s. Verbrauchter Schluessel von %s bis %s." $AktionText $Startwert $Stoppwert]
		}
	}
	return $Meldung
}

proc EingabeZufallsgeneratorPruefen {Zahl} {
	# prueft, ob die Zahl 10-stellig ist
	set Laenge [string length $Zahl]
	.top3.frAlles.lbLaenge configure -text [::msgcat::mc "Laenge: %s" $Laenge]
	if {$Laenge == 10} {
		.top3.frAlles.btOK configure -state normal
	} else {
		.top3.frAlles.btOK configure -state disabled
	}
	update idletask
}

proc PruefenObZiffer {Text Zeichen Index} {
	# Nur Ziffern sind zulaessig.
	# Die erste Ziffer darf nicht 0 sein.
	set Ergebnis 0
	if {[string is integer $Zeichen]} {
		if {$Index >= 1 || ($Index == 0 && $Zeichen >= 1 && $Zeichen <=9)} {
			set Ergebnis 1
		}
	}
	if {[string length $Text] > 10} {
		set Ergebnis 0
	}
	return $Ergebnis
}

proc DialogZufallsgeneratorInitialisieren {} {
	# Zehnstelligen Zahl zur Initialisierung des Zufallszahlengenerators eingeben
	global ZufallgeneratorIstInitialisiert
	global ZufallsgeneratorInitialisierenZahl
	
	set ZufallsgeneratorInitialisierenZahl ""
	toplevel .top3
	wm title .top3 [::msgcat::mc "Initialisierung"]
	ttk::frame .top3.frAlles
	ttk::label .top3.frAlles.lbWarnung -wraplength 300 -justify left -text [::msgcat::mc "Warnung: Die von einem Computer erzeugten Zufallszahlen sind nur Pseudo-Zufallszahlen. Die verschluesselte Datei kann deshalb bei genuegend Rechenleistung von einem Angreifer entschluesselt werden. Verwenden Sie besser eine eigene Schluesseldatei mit Zufallszahlen."]
	ttk::label .top3.frAlles.lbEingabe -text [::msgcat::mc "Bitte eine 10-stellige Zufallszahl eingeben:"]
	entry .top3.frAlles.enEingabe -textvariable ZufallsgeneratorInitialisierenZahl -validate key -vcmd {PruefenObZiffer %P %S %i}
	ttk::label .top3.frAlles.lbLaenge -text ""
	ttk::button .top3.frAlles.btOK -text [::msgcat::mc "OK"] -state disabled -command {
		expr srand($ZufallsgeneratorInitialisierenZahl)
		set ZufallgeneratorIstInitialisiert 1
		grab release .top3
		destroy .top3
	}
	bind .top3.frAlles.enEingabe <KeyRelease> {EingabeZufallsgeneratorPruefen $ZufallsgeneratorInitialisierenZahl}
	pack .top3.frAlles -expand yes -fill both
	pack .top3.frAlles.lbWarnung -padx 3 -pady 3 -anchor ne
	pack .top3.frAlles.lbEingabe -padx 3 -pady 3
	pack .top3.frAlles.enEingabe -padx 3 -pady 3
	pack .top3.frAlles.lbLaenge -padx 3 -pady 3
	pack .top3.frAlles.btOK -padx 3 -pady 3
	focus .top3.frAlles.enEingabe
	update idletask	
	tkwait visibility .top3
	grab set .top3
}

proc SchluesselErzeugen {EingabeDateiText Eingabedatei Dateiendung HomeOrdner} {
	# Erstellt eine Schluesseldatei
	
	global Schluesseldatei
	global ZufallgeneratorIstInitialisiert
	
	set ZufallgeneratorIstInitialisiert 0 

	DialogZufallsgeneratorInitialisieren
	tkwait window .top3

	if {$ZufallgeneratorIstInitialisiert == 0} {
		set Meldung [::msgcat::mc "Der Schluessel wurde nicht intialisiert."]
		return $Meldung
	}
	
	set Schluesseldatei ""

	if {$EingabeDateiText eq "Datei"} {
		if {$Eingabedatei eq "" || [file isfile $Eingabedatei] == 0} {
			set Meldung [::msgcat::mc "Waehlen Sie eine zuerst eine Datei aus."]
			return $Meldung
		} 
	}

	if {$EingabeDateiText eq "Datei"} {
		set Schluesseldatei $Eingabedatei$Dateiendung
	} else {
		set Schluesseldatei [file join $HomeOrdner "[clock clicks -milliseconds]$Dateiendung"]
	}
	if {[file exist $Schluesseldatei] == 1} {
		set Meldung [::msgcat::mc "Der Schluessel %s existiert schon." $Schluesseldatei]
		return $Meldung
	}
	
	if {$EingabeDateiText eq "Datei"} {
		set Dateigroesse [file size $Eingabedatei]
	} else {
		set Dateigroesse [.frAlles.frText.tx count -chars 1.0 end]
	}

	set FortschrittGrenze [expr $Dateigroesse/100]
	set FortschrittZaehler 0
	set FortschrittZaehler2 0	

	# Schluessel erstellen
	set fSchluessel [open $Schluesseldatei w]
	fconfigure $fSchluessel -translation binary
	for {set i 0} {$i <= $Dateigroesse} {incr i} {
		set CodeDezimal [expr int(rand()*256)]
		set CodeBinaer [format %08b $CodeDezimal]
		set Ausgabe [binary format B8 $CodeBinaer]
		puts -nonewline $fSchluessel $Ausgabe
		
		# Fortschrittsbalken aktualisieren
		incr FortschrittZaehler
		if {$FortschrittZaehler >= $FortschrittGrenze} {
			incr FortschrittZaehler2
			.frAlles.pb configure -value $FortschrittZaehler2
			update idletask
			set FortschrittZaehler 0
		}
	}
	close $fSchluessel
	
	SchluessellisteErgaenzen $Schluesseldatei
	
	set Meldung [::msgcat::mc "Der Schluessel %s wurde erstellt." $Schluesseldatei]
	return $Meldung
}

proc ListeDuplikateEntfernen {Liste} {
	# Entfernt aus der Liste die Duplikate, ohne die Reihenfolge der Listenelemente zu aendern
	for {set i 0} {$i < [llength $Liste]} {incr i} {
		set Indizes [lsearch -all $Liste [lindex $Liste $i]]
		for {set j [expr [llength $Indizes] - 1]} {$j > 0} {incr j -1} {
			# die doppelten Listenelemente vom Ende aus entfernen
			set Index [lindex $Indizes $j]
			set Liste [lreplace $Liste $Index $Index]
		}
	}
	return $Liste
}

proc SchluessellisteErgaenzen {Schluesseldatei} {
	# Ergaenzt die Schluesselliste um einen neuen Eintrag
	# Entfernt Dubletten und sortiert die Liste alphabetisch
	global Schluesselliste
	global SchluessellisteMaxAnzahl

	if {[llength $Schluesselliste] >= $SchluessellisteMaxAnzahl} {
		set Schluesselliste [lrange $Schluesselliste 1 end]
	}
	lappend Schluesselliste $Schluesseldatei
	set Schluesselliste [ListeDuplikateEntfernen $Schluesselliste]
	#set Schluesselliste [lsort -unique $Schluesselliste] ; # die Sortierung weglassen, weil sonst immer der erste Schluessel geloescht wird, wenn SchluessellisteMaxAnzahl erreicht wurde
	.frAlles.cbSchluesseldatei configure -values $Schluesselliste
	update idletasks
}

proc NurGanzzahlen {Zeichen} {
	return [string is integer $Zeichen]
}

# *** Hauptprogramm ***

if {$tclexecomp == 1} {
	set Konfigurationsdatei [file join [file dirname [file normalize [info nameofexecutable]]] Konf.txt] 
} else {
	set Konfigurationsdatei [file join [file dirname [file normalize [info script]]] Konf.txt] 
}
set HomeOrdner $env(HOME)
set Sprache "Deutsch"
set Meldung ""
set Suchbegriff ""
set Dateiendung ".otp" ; # Dateiendung fuer verschluesselte Dateien
set Textkennung "*OTP*" ; # Kennzeichnung, dass es sich um einen verschluesselten Text handelt
set ZufallgeneratorIstInitialisiert 0 ; # Vermerkt, ob der Zufallgenerator initialisiert wurde
set ZufallsgeneratorInitialisierenZahl "" ; # Zahl zum Initialisieren des Zufallgenerators
set SchluessellisteMaxAnzahl 10 ; # Maximale Anzahl an Eintraegen in der Liste mit den Schluesseln

# wenn keine drei Parameter uebergeben werden, wird die GUI angezeigt
if {$argc == 3} {
	set Eingabedatei [lindex $argv 0]
	set Schluesseldatei [lindex $argv 1]
	set Startwert [lindex $argv 2]
	set EingabeDateiText "Datei"
	set AusgabeDateiText "Datei"
	set HistogrammJaNein "nein"
	set Konsole 1
	Start $EingabeDateiText $AusgabeDateiText $Eingabedatei $Schluesseldatei $Startwert $Dateiendung $Textkennung $HomeOrdner $HistogrammJaNein $Konsole
	exit
} else {
	set Eingabedatei [::msgcat::mc "bitte Datei waehlen"]
	set Schluesseldatei [::msgcat::mc "bitte Datei waehlen"]
	set Startwert 0
	set EingabeDateiText "Datei"
	set AusgabeDateiText "Datei"
	set HistogrammJaNein "ja"
	set Konsole 0
	set Schluesselliste {}
	if {[file exist $Konfigurationsdatei]} {
		set f [open $Konfigurationsdatei r]
		while {[gets $f Zeile] >= 0} {
			set Nummer [string rang $Zeile 0 0]
			set Wert [string rang $Zeile 2 end]
			switch $Nummer {
				1 {set EingabeDateiText $Wert}
				2 {set AusgabeDateiText $Wert}
				3 {set Eingabedatei $Wert}
				4 {set Schluesseldatei $Wert}
				5 {set HistogrammJaNein $Wert}
				6 {set Sprache $Wert}
				7 {set Schriftgroesse $Wert}
				8 {
					if {$Wert ne ""} {
						lappend Schluesselliste $Wert
					}
				}
			}
		}
		close $f
	}

	font configure GlobSchrift -size $Schriftgroesse

	wm title . [::msgcat::mc "%s" $ProgrammTitel]

	option add *Menu.tearOff 0
	. configure -menu .m
	menu .m
	.m add command -label [::msgcat::mc "Schliessen"] -font GlobSchrift -command ProgrammBeenden
	.m add cascade -label [::msgcat::mc "Sprache"] -font GlobSchrift -menu .m.sprache
	.m add cascade -label [::msgcat::mc "Schriftgroesse"] -font GlobSchrift -menu .m.schriftgroesse
	.m add command -label [::msgcat::mc "Info"] -font GlobSchrift -command {InfoAnzeigen $ProgrammName $ProgrammTitel $ProgrammVersion}
	
	menu .m.sprache
	.m.sprache add radiobutton -label "Deutsch" -font GlobSchrift -variable Sprache -command {SpracheAendern $Sprache}
	.m.sprache add radiobutton -label "English" -font GlobSchrift -variable Sprache -command {SpracheAendern $Sprache}

	menu .m.schriftgroesse
	.m.schriftgroesse add radiobutton -label 8 -variable Schriftgroesse -value 8 -underline 0 -font GlobSchrift -command {font configure GlobSchrift -size $Schriftgroesse}
	.m.schriftgroesse add radiobutton -label 9 -variable Schriftgroesse -value 9 -underline 0 -font GlobSchrift -command {font configure GlobSchrift -size $Schriftgroesse}
	.m.schriftgroesse add radiobutton -label 10 -variable Schriftgroesse -value 10 -underline 0 -font GlobSchrift -command {font configure GlobSchrift -size $Schriftgroesse}
	.m.schriftgroesse add radiobutton -label 11 -variable Schriftgroesse -value 11 -underline 0 -font GlobSchrift -command {font configure GlobSchrift -size $Schriftgroesse}
	.m.schriftgroesse add radiobutton -label 12 -variable Schriftgroesse -value 12 -underline 0 -font GlobSchrift -command {font configure GlobSchrift -size $Schriftgroesse}
	.m.schriftgroesse add radiobutton -label 14 -variable Schriftgroesse -value 14 -underline 0 -font GlobSchrift -command {font configure GlobSchrift -size $Schriftgroesse}
	.m.schriftgroesse add radiobutton -label 16 -variable Schriftgroesse -value 16 -underline 0 -font GlobSchrift -command {font configure GlobSchrift -size $Schriftgroesse}
	.m.schriftgroesse add radiobutton -label 18 -variable Schriftgroesse -value 18 -underline 0 -font GlobSchrift -command {font configure GlobSchrift -size $Schriftgroesse}
	.m.schriftgroesse add radiobutton -label 20 -variable Schriftgroesse -value 20 -underline 0 -font GlobSchrift -command {font configure GlobSchrift -size $Schriftgroesse}
	.m.schriftgroesse add radiobutton -label 22 -variable Schriftgroesse -value 22 -underline 0 -font GlobSchrift -command {font configure GlobSchrift -size $Schriftgroesse}

	ttk::frame .frAlles

	ttk::label .frAlles.lbEingabe -text [::msgcat::mc "Eingabe:"]
	ttk::frame .frAlles.frEingabe
	ttk::radiobutton .frAlles.frEingabe.rbEingabeDatei -text [::msgcat::mc "Datei"] -variable EingabeDateiText -value "Datei" -command {DateiTextAuswaehlen "Datei" $AusgabeDateiText}
	ttk::radiobutton .frAlles.frEingabe.rbEingabeText -text [::msgcat::mc "Text"] -variable EingabeDateiText -value "Text" -command {DateiTextAuswaehlen "Text" $AusgabeDateiText}
	
	ttk::label .frAlles.lbAusgabe -text [::msgcat::mc "Ausgabe:"]
	ttk::frame .frAlles.frAusgabe
	ttk::radiobutton .frAlles.frAusgabe.rbAusgabeDatei -text [::msgcat::mc "Datei"] -variable AusgabeDateiText -value "Datei" -command {DateiTextAuswaehlen $EingabeDateiText "Datei"}
	ttk::radiobutton .frAlles.frAusgabe.rbAusgabeText -text [::msgcat::mc "Text"] -variable AusgabeDateiText -value "Text" -command {DateiTextAuswaehlen $EingabeDateiText "Text"}
	
	ttk::label .frAlles.lb1Eingabedatei -text [::msgcat::mc "Datei:"]
	ttk::label .frAlles.lb2Eingabedatei -textvariable Eingabedatei
	ttk::button .frAlles.btEingabedatei -text "..." -command {set Eingabedatei [tk_getOpenFile]}

	ttk::label .frAlles.lbSchluesseldatei -text [::msgcat::mc "Schluessel:"]
	ttk::combobox .frAlles.cbSchluesseldatei -state readonly -width 60 -values $Schluesselliste -textvariable Schluesseldatei -font GlobSchrift
	ttk::button .frAlles.btSchluesseldatei -text "..." -command {
		set tmp $Schluesseldatei
		set Schluesseldatei [tk_getOpenFile]
		if {$Schluesseldatei ne ""} {
			SchluessellisteErgaenzen $Schluesseldatei
		} else {
			set Schluesseldatei $tmp
		}
	}
	ttk::button .frAlles.btSchluesselErzeugen -text [::msgcat::mc "Erzeugen"] -command {
		set Meldung [SchluesselErzeugen $EingabeDateiText $Eingabedatei ".key" $HomeOrdner]
		set ZufallsgeneratorInitialisierenZahl ""
		update idletask
	}

	ttk::label .frAlles.lbStartwert -text [::msgcat::mc "Starte bei Byte:"]
	ttk::entry .frAlles.enStartwert -textvariable Startwert -font GlobSchrift -validatecommand {NurGanzzahlen %S} -validate key

	ttk::label .frAlles.lbHistogramm -text [::msgcat::mc "Histogramm:"]
	ttk::frame .frAlles.frHistogramm
	ttk::radiobutton .frAlles.frHistogramm.rbHistogrammJa -text [::msgcat::mc "ja"] -variable HistogrammJaNein -value "ja"
	ttk::radiobutton .frAlles.frHistogramm.rbHistogrammNein -text [::msgcat::mc "nein"] -variable HistogrammJaNein -value "nein"
	
	ttk::button .frAlles.btStart -text [::msgcat::mc "Start"] -command {
		set Meldung ""
		set Meldung [Start $EingabeDateiText $AusgabeDateiText $Eingabedatei $Schluesseldatei $Startwert $Dateiendung $Textkennung $HomeOrdner $HistogrammJaNein $Konsole]
		set Startwert 0
	}

	ttk::label .frAlles.lbFortschritt -text [::msgcat::mc "Fortschritt:"]
	ttk::progressbar .frAlles.pb -orient horizontal -maximum 100 -length 400 -value 0
	
	ttk::label .frAlles.lb1Meldung -text [::msgcat::mc "Meldung:"]
	ttk::label .frAlles.lb2Meldung -textvariable Meldung

	ttk::label .frAlles.lbSuchen -text [::msgcat::mc "Suchbegriff:"]
	ttk::entry .frAlles.enSuchen -textvariable Suchbegriff -font GlobSchrift
	ttk::button .frAlles.btSuchen -text [::msgcat::mc "Suchen"] -command {Suchen $Suchbegriff}

	ttk::label .frAlles.lbText -text [::msgcat::mc "Text:"]

	ttk::frame .frAlles.frText
	text .frAlles.frText.tx -height 5 -width 15 -wrap word -font GlobSchrift -xscrollcommand {.frAlles.frText.sbX set} -yscrollcommand {.frAlles.frText.sbY set}
	ttk::scrollbar .frAlles.frText.sbX -orient horizontal -command {.frAlles.frText.tx xview}
	ttk::scrollbar .frAlles.frText.sbY -command {.frAlles.frText.tx yview}

	pack .frAlles -side top -expand yes -fill both

	grid .frAlles.lbEingabe -row 0 -column 0 -padx 3 -pady 3 -sticky w
	grid .frAlles.frEingabe -row 0 -column 1 -padx 3 -pady 3 -sticky w
	
		grid .frAlles.frEingabe.rbEingabeDatei -row 0 -column 0 -padx 3 -pady 3 -sticky w
		grid .frAlles.frEingabe.rbEingabeText -row 0 -column 1 -padx 3 -pady 3 -sticky w

	grid .frAlles.lbAusgabe -row 1 -column 0 -padx 3 -pady 3 -sticky w
	grid .frAlles.frAusgabe -row 1 -column 1 -padx 3 -pady 3 -sticky w

		grid .frAlles.frAusgabe.rbAusgabeDatei -row 1 -column 0 -padx 3 -pady 3 -sticky w
		grid .frAlles.frAusgabe.rbAusgabeText -row 1 -column 1 -padx 3 -pady 3 -sticky w

	grid .frAlles.lb1Eingabedatei -row 2 -column 0 -padx 3 -pady 3 -sticky w
	grid .frAlles.lb2Eingabedatei -row 2 -column 1 -columnspan 2 -padx 3 -pady 3 -sticky w
	grid .frAlles.btEingabedatei -row 2 -column 3 -padx 3 -pady 3 -sticky w

	grid .frAlles.lbSchluesseldatei -row 3 -column 0 -padx 3 -pady 3 -sticky w
	grid .frAlles.cbSchluesseldatei -row 3 -column 1 -columnspan 2 -padx 3 -pady 3 -sticky we
	grid .frAlles.btSchluesseldatei -row 3 -column 3 -padx 3 -pady 3 -sticky w
	grid .frAlles.btSchluesselErzeugen -row 3 -column 4 -padx 3 -pady 3 -sticky w

	grid .frAlles.lbStartwert -row 4 -column 0 -padx 3 -pady 3 -sticky w
	grid .frAlles.enStartwert -row 4 -column 1 -padx 3 -pady 3 -sticky we

	grid .frAlles.lbHistogramm -row 5 -column 0 -padx 3 -pady 3 -sticky w
	grid .frAlles.frHistogramm -row 5 -column 1 -padx 3 -pady 3 -sticky w
	
		grid .frAlles.frHistogramm.rbHistogrammJa -row 0 -column 0 -padx 3 -pady 3 -sticky w
		grid .frAlles.frHistogramm.rbHistogrammNein -row 0 -column 1 -padx 3 -pady 3 -sticky w

	grid .frAlles.btStart -row 6 -column 0 -padx 3 -pady 3

	grid .frAlles.lbFortschritt -row 7 -column 0 -padx 3 -pady 3 -sticky w
	grid .frAlles.pb -row 7 -column 1 -columnspan 3 -padx 3 -pady 3 -sticky w

	grid .frAlles.lb1Meldung -row 8 -column 0 -padx 3 -pady 3 -sticky w
	grid .frAlles.lb2Meldung -row 8 -column 1 -columnspan 3 -padx 3 -pady 3 -sticky w

	grid .frAlles.lbText -row 9 -column 0 -padx 3 -pady 3 -sticky nw
	grid .frAlles.frText -row 9 -column 1 -columnspan 4 -sticky nesw

	grid .frAlles.lbSuchen -row 10 -column 0 -padx 3 -pady 3 -sticky w
	grid .frAlles.enSuchen -row 10 -column 1 -padx 3 -pady 3 -sticky we
	grid .frAlles.btSuchen -row 10 -column 2 -padx 3 -pady 3 -sticky w
	
	pack .frAlles.frText.sbX -side bottom -fill x
	pack .frAlles.frText.sbY -side right -fill y
	pack .frAlles.frText.tx -side top -expand yes -fill both

	grid rowconfigure .frAlles 9 -weight 1
	grid columnconfigure .frAlles 1 -weight 1 ; #3

	# Voreinstellung: versteckte Dateien ausblenden
	#catch {tk_getOpenFile foo bar}
	#set ::tk::dialog::file::showHiddenVar 0
	#set ::tk::dialog::file::showHiddenBtn 1

	DateiTextAuswaehlen $EingabeDateiText $AusgabeDateiText
	::tooltip::tooltip on
	SpracheAendern $Sprache
	DateiTextAuswaehlen $EingabeDateiText $AusgabeDateiText
}
